// src/index.ts
import crypto from "crypto";
import { Chacha20 } from "ts-chacha20";
import C from "crypto-js";
import argon from "argon2-browser";
function pbkdf2({ password, salt }) {
  console.log(salt, Buffer.from(salt).toString("hex"));
  const result = C.PBKDF2(password, Buffer.from(salt).toString("hex"), {
    keySize: 512 / 32,
    iterations: 1,
    hasher: C.algo.SHA256
  }).toString(C.enc.Base64);
  return Buffer.from(result, "base64");
}
async function argon2({ password, salt }) {
  const result = await argon.hash({
    pass: password,
    salt,
    hashLen: 32,
    time: 2,
    mem: 32,
    parallelism: 1,
    type: argon.ArgonType.Argon2d
  });
  return Buffer.from(result.hash);
}
function generateRandom(length = 16) {
  return Uint8Array.from(crypto.randomBytes(length));
}
async function aesEncrypt(data, password) {
  const salt = generateRandom(12);
  const key = await argon2({ password, salt });
  const encrypt = new Chacha20(key.subarray(0, 32), salt).encrypt(
    Buffer.from(data, "utf8")
  );
  const packaged = Buffer.concat([salt, encrypt]);
  return packaged.toString("base64");
}
async function aesDecrypt(data, password) {
  const unpackaged = Buffer.from(data, "base64");
  const salt = unpackaged.subarray(0, 12);
  const input = unpackaged.subarray(12, data.length);
  const key = await argon2({ password, salt });
  const message = new Chacha20(key.subarray(0, 32), salt).decrypt(input);
  return Buffer.from(message).toString("utf8");
}
export {
  aesDecrypt,
  aesEncrypt,
  argon2,
  pbkdf2
};
